import numpy as np
import unittest

from flood_fill_filter.linear_srgb_conv import *


class TestLinearSrgbConv(unittest.TestCase):
    RAINBOW_SRGB = np.array([[[0., 0., 0., 255.],
                              [2., 2., 2., 255.],
                              [11., 10., 10., 255.],
                              [18., 19., 19., 255.],
                              [27., 27., 26., 255.],
                              [35., 35., 35., 255.],
                              [43., 43., 43., 255.],
                              [52., 52., 51., 255.],
                              [60., 60., 60., 255.],
                              [69., 69., 68., 255.],
                              [77., 76., 76., 255.],
                              [85., 85., 85., 255.],
                              [93., 93., 94., 255.],
                              [102., 101., 102., 255.],
                              [110., 109., 110., 255.],
                              [118., 118., 118., 255.],
                              [126., 126., 126., 255.],
                              [135., 134., 134., 255.],
                              [142., 143., 143., 255.],
                              [151., 151., 151., 255.],
                              [159., 160., 159., 255.],
                              [168., 168., 167., 255.],
                              [176., 176., 176., 255.],
                              [184., 184., 185., 255.],
                              [192., 193., 193., 255.],
                              [201., 201., 200., 255.],
                              [209., 209., 209., 255.],
                              [217., 217., 218., 255.],
                              [226., 226., 226., 255.],
                              [234., 234., 233., 255.],
                              [242., 242., 242., 255.],
                              [250., 251., 250., 255.]],

                             [[255., 128., 128., 255.],
                              [255., 144., 128., 255.],
                              [255., 169., 128., 255.],
                              [255., 193., 128., 255.],
                              [255., 217., 128., 255.],
                              [255., 242., 128., 255.],
                              [243., 255., 128., 255.],
                              [219., 255., 128., 255.],
                              [195., 255., 128., 255.],
                              [170., 255., 128., 255.],
                              [145., 255., 128., 255.],
                              [128., 255., 133., 255.],
                              [128., 255., 158., 255.],
                              [128., 255., 182., 255.],
                              [128., 255., 207., 255.],
                              [128., 255., 231., 255.],
                              [128., 255., 255., 255.],
                              [128., 230., 255., 255.],
                              [128., 205., 255., 255.],
                              [128., 181., 255., 255.],
                              [128., 156., 255., 255.],
                              [128., 132., 255., 255.],
                              [147., 128., 255., 255.],
                              [171., 128., 255., 255.],
                              [197., 128., 255., 255.],
                              [221., 128., 255., 255.],
                              [245., 128., 255., 255.],
                              [255., 128., 241., 255.],
                              [255., 128., 216., 255.],
                              [255., 128., 191., 255.],
                              [255., 128., 167., 255.],
                              [255., 128., 142., 255.]],

                             [[255., 0., 0., 255.],
                              [255., 34., 0., 255.],
                              [255., 83., 0., 255.],
                              [255., 132., 0., 255.],
                              [255., 180., 0., 255.],
                              [255., 229., 0., 255.],
                              [232., 255., 0., 255.],
                              [184., 255., 0., 255.],
                              [134., 255., 0., 255.],
                              [86., 255., 0., 255.],
                              [36., 255., 0., 255.],
                              [0., 255., 12., 255.],
                              [0., 255., 61., 255.],
                              [0., 255., 109., 255.],
                              [0., 255., 158., 255.],
                              [0., 255., 208., 255.],
                              [0., 254., 255., 255.],
                              [0., 205., 255., 255.],
                              [0., 156., 255., 255.],
                              [0., 107., 255., 255.],
                              [0., 58., 255., 255.],
                              [0., 9., 255., 255.],
                              [40., 0., 255., 255.],
                              [88., 0., 255., 255.],
                              [138., 0., 255., 255.],
                              [186., 0., 255., 255.],
                              [235., 0., 255., 255.],
                              [255., 0., 226., 255.],
                              [255., 0., 177., 255.],
                              [255., 0., 128., 255.],
                              [255., 0., 79., 255.],
                              [255., 0., 30., 255.]],

                             [[142., 0., 0., 255.],
                              [142., 19., 0., 255.],
                              [142., 46., 0., 255.],
                              [142., 74., 0., 255.],
                              [142., 101., 0., 255.],
                              [142., 128., 0., 255.],
                              [130., 142., 0., 255.],
                              [103., 142., 0., 255.],
                              [75., 142., 0., 255.],
                              [48., 142., 0., 255.],
                              [20., 142., 0., 255.],
                              [0., 142., 7., 255.],
                              [0., 142., 34., 255.],
                              [0., 142., 61., 255.],
                              [0., 142., 88., 255.],
                              [0., 142., 116., 255.],
                              [0., 142., 142., 255.],
                              [0., 115., 142., 255.],
                              [0., 87., 142., 255.],
                              [0., 60., 142., 255.],
                              [0., 32., 142., 255.],
                              [0., 5., 142., 255.],
                              [22., 0., 142., 255.],
                              [49., 0., 142., 255.],
                              [77., 0., 142., 255.],
                              [104., 0., 142., 255.],
                              [131., 0., 142., 255.],
                              [142., 0., 126., 255.],
                              [142., 0., 99., 255.],
                              [142., 0., 71., 255.],
                              [142., 0., 44., 255.],
                              [142., 0., 17., 255.]],

                             [[0., 0., 0., 191.],
                              [2., 2., 2., 191.],
                              [11., 10., 10., 191.],
                              [18., 19., 19., 191.],
                              [27., 27., 26., 191.],
                              [35., 35., 35., 191.],
                              [43., 43., 43., 191.],
                              [52., 52., 51., 191.],
                              [60., 60., 60., 191.],
                              [69., 69., 68., 191.],
                              [77., 76., 76., 191.],
                              [85., 85., 85., 191.],
                              [93., 93., 94., 191.],
                              [102., 101., 102., 191.],
                              [110., 109., 110., 191.],
                              [118., 118., 118., 191.],
                              [126., 126., 126., 191.],
                              [135., 134., 134., 191.],
                              [142., 143., 143., 191.],
                              [151., 151., 151., 191.],
                              [159., 160., 159., 191.],
                              [168., 168., 167., 191.],
                              [176., 176., 176., 191.],
                              [184., 184., 185., 191.],
                              [192., 193., 193., 191.],
                              [201., 201., 200., 191.],
                              [209., 209., 209., 191.],
                              [217., 217., 218., 191.],
                              [226., 226., 226., 191.],
                              [234., 234., 233., 191.],
                              [242., 242., 242., 191.],
                              [250., 251., 250., 191.]],

                             [[255., 128., 128., 191.],
                              [255., 144., 128., 191.],
                              [255., 169., 128., 191.],
                              [255., 193., 128., 191.],
                              [255., 217., 128., 191.],
                              [255., 242., 128., 191.],
                              [243., 255., 128., 191.],
                              [219., 255., 128., 191.],
                              [195., 255., 128., 191.],
                              [170., 255., 128., 191.],
                              [145., 255., 128., 191.],
                              [128., 255., 133., 191.],
                              [128., 255., 158., 191.],
                              [128., 255., 182., 191.],
                              [128., 255., 207., 191.],
                              [128., 255., 231., 191.],
                              [128., 255., 255., 191.],
                              [128., 230., 255., 191.],
                              [128., 205., 255., 191.],
                              [128., 181., 255., 191.],
                              [128., 156., 255., 191.],
                              [128., 132., 255., 191.],
                              [147., 128., 255., 191.],
                              [171., 128., 255., 191.],
                              [197., 128., 255., 191.],
                              [221., 128., 255., 191.],
                              [245., 128., 255., 191.],
                              [255., 128., 241., 191.],
                              [255., 128., 216., 191.],
                              [255., 128., 191., 191.],
                              [255., 128., 167., 191.],
                              [255., 128., 142., 191.]],

                             [[255., 0., 0., 191.],
                              [255., 34., 0., 191.],
                              [255., 83., 0., 191.],
                              [255., 132., 0., 191.],
                              [255., 180., 0., 191.],
                              [255., 229., 0., 191.],
                              [232., 255., 0., 191.],
                              [184., 255., 0., 191.],
                              [134., 255., 0., 191.],
                              [86., 255., 0., 191.],
                              [36., 255., 0., 191.],
                              [0., 255., 12., 191.],
                              [0., 255., 61., 191.],
                              [0., 255., 109., 191.],
                              [0., 255., 158., 191.],
                              [0., 255., 208., 191.],
                              [0., 254., 255., 191.],
                              [0., 205., 255., 191.],
                              [0., 156., 255., 191.],
                              [0., 107., 255., 191.],
                              [0., 58., 255., 191.],
                              [0., 9., 255., 191.],
                              [40., 0., 255., 191.],
                              [88., 0., 255., 191.],
                              [138., 0., 255., 191.],
                              [186., 0., 255., 191.],
                              [235., 0., 255., 191.],
                              [255., 0., 226., 191.],
                              [255., 0., 177., 191.],
                              [255., 0., 128., 191.],
                              [255., 0., 79., 191.],
                              [255., 0., 30., 191.]],

                             [[142., 0., 0., 191.],
                              [142., 19., 0., 191.],
                              [142., 46., 0., 191.],
                              [142., 74., 0., 191.],
                              [142., 101., 0., 191.],
                              [142., 128., 0., 191.],
                              [130., 142., 0., 191.],
                              [103., 142., 0., 191.],
                              [75., 142., 0., 191.],
                              [48., 142., 0., 191.],
                              [20., 142., 0., 191.],
                              [0., 142., 7., 191.],
                              [0., 142., 34., 191.],
                              [0., 142., 61., 191.],
                              [0., 142., 88., 191.],
                              [0., 142., 116., 191.],
                              [0., 142., 142., 191.],
                              [0., 115., 142., 191.],
                              [0., 87., 142., 191.],
                              [0., 60., 142., 191.],
                              [0., 32., 142., 191.],
                              [0., 5., 142., 191.],
                              [22., 0., 142., 191.],
                              [49., 0., 142., 191.],
                              [77., 0., 142., 191.],
                              [104., 0., 142., 191.],
                              [131., 0., 142., 191.],
                              [142., 0., 126., 191.],
                              [142., 0., 99., 191.],
                              [142., 0., 71., 191.],
                              [142., 0., 44., 191.],
                              [142., 0., 17., 191.]],

                             [[0., 0., 0., 127.],
                              [2., 2., 2., 127.],
                              [11., 10., 10., 127.],
                              [18., 19., 19., 127.],
                              [27., 27., 26., 127.],
                              [35., 35., 35., 127.],
                              [43., 43., 43., 127.],
                              [52., 52., 51., 127.],
                              [60., 60., 60., 127.],
                              [69., 69., 68., 127.],
                              [77., 76., 76., 127.],
                              [85., 85., 85., 127.],
                              [93., 93., 94., 127.],
                              [102., 101., 102., 127.],
                              [110., 109., 110., 127.],
                              [118., 118., 118., 127.],
                              [126., 126., 126., 127.],
                              [135., 134., 134., 127.],
                              [142., 143., 143., 127.],
                              [151., 151., 151., 127.],
                              [159., 160., 159., 127.],
                              [168., 168., 167., 127.],
                              [176., 176., 176., 127.],
                              [184., 184., 185., 127.],
                              [192., 193., 193., 127.],
                              [201., 201., 200., 127.],
                              [209., 209., 209., 127.],
                              [217., 217., 218., 127.],
                              [226., 226., 226., 127.],
                              [234., 234., 233., 127.],
                              [242., 242., 242., 127.],
                              [250., 251., 250., 127.]],

                             [[255., 128., 128., 127.],
                              [255., 144., 128., 127.],
                              [255., 169., 128., 127.],
                              [255., 193., 128., 127.],
                              [255., 217., 128., 127.],
                              [255., 242., 128., 127.],
                              [243., 255., 128., 127.],
                              [219., 255., 128., 127.],
                              [195., 255., 128., 127.],
                              [170., 255., 128., 127.],
                              [145., 255., 128., 127.],
                              [128., 255., 133., 127.],
                              [128., 255., 158., 127.],
                              [128., 255., 182., 127.],
                              [128., 255., 207., 127.],
                              [128., 255., 231., 127.],
                              [128., 255., 255., 127.],
                              [128., 230., 255., 127.],
                              [128., 205., 255., 127.],
                              [128., 181., 255., 127.],
                              [128., 156., 255., 127.],
                              [128., 132., 255., 127.],
                              [147., 128., 255., 127.],
                              [171., 128., 255., 127.],
                              [197., 128., 255., 127.],
                              [221., 128., 255., 127.],
                              [245., 128., 255., 127.],
                              [255., 128., 241., 127.],
                              [255., 128., 216., 127.],
                              [255., 128., 191., 127.],
                              [255., 128., 167., 127.],
                              [255., 128., 142., 127.]],

                             [[255., 0., 0., 127.],
                              [255., 34., 0., 127.],
                              [255., 83., 0., 127.],
                              [255., 132., 0., 127.],
                              [255., 180., 0., 127.],
                              [255., 229., 0., 127.],
                              [232., 255., 0., 127.],
                              [184., 255., 0., 127.],
                              [134., 255., 0., 127.],
                              [86., 255., 0., 127.],
                              [36., 255., 0., 127.],
                              [0., 255., 12., 127.],
                              [0., 255., 61., 127.],
                              [0., 255., 109., 127.],
                              [0., 255., 158., 127.],
                              [0., 255., 208., 127.],
                              [0., 254., 255., 127.],
                              [0., 205., 255., 127.],
                              [0., 156., 255., 127.],
                              [0., 107., 255., 127.],
                              [0., 58., 255., 127.],
                              [0., 9., 255., 127.],
                              [40., 0., 255., 127.],
                              [88., 0., 255., 127.],
                              [138., 0., 255., 127.],
                              [186., 0., 255., 127.],
                              [235., 0., 255., 127.],
                              [255., 0., 226., 127.],
                              [255., 0., 177., 127.],
                              [255., 0., 128., 127.],
                              [255., 0., 79., 127.],
                              [255., 0., 30., 127.]],

                             [[142., 0., 0., 127.],
                              [142., 19., 0., 127.],
                              [142., 46., 0., 127.],
                              [142., 74., 0., 127.],
                              [142., 101., 0., 127.],
                              [142., 128., 0., 127.],
                              [130., 142., 0., 127.],
                              [103., 142., 0., 127.],
                              [75., 142., 0., 127.],
                              [48., 142., 0., 127.],
                              [20., 142., 0., 127.],
                              [0., 142., 7., 127.],
                              [0., 142., 34., 127.],
                              [0., 142., 61., 127.],
                              [0., 142., 88., 127.],
                              [0., 142., 116., 127.],
                              [0., 142., 142., 127.],
                              [0., 115., 142., 127.],
                              [0., 87., 142., 127.],
                              [0., 60., 142., 127.],
                              [0., 32., 142., 127.],
                              [0., 5., 142., 127.],
                              [22., 0., 142., 127.],
                              [49., 0., 142., 127.],
                              [77., 0., 142., 127.],
                              [104., 0., 142., 127.],
                              [131., 0., 142., 127.],
                              [142., 0., 126., 127.],
                              [142., 0., 99., 127.],
                              [142., 0., 71., 127.],
                              [142., 0., 44., 127.],
                              [142., 0., 17., 127.]],

                             [[0., 0., 0., 63.],
                              [2., 2., 2., 63.],
                              [11., 10., 10., 63.],
                              [18., 19., 19., 63.],
                              [27., 27., 26., 63.],
                              [35., 35., 35., 63.],
                              [43., 43., 43., 63.],
                              [52., 52., 51., 63.],
                              [60., 60., 60., 63.],
                              [69., 69., 68., 63.],
                              [77., 76., 76., 63.],
                              [85., 85., 85., 63.],
                              [93., 93., 94., 63.],
                              [102., 101., 102., 63.],
                              [110., 109., 110., 63.],
                              [118., 118., 118., 63.],
                              [126., 126., 126., 63.],
                              [135., 134., 134., 63.],
                              [142., 143., 143., 63.],
                              [151., 151., 151., 63.],
                              [159., 160., 159., 63.],
                              [168., 168., 167., 63.],
                              [176., 176., 176., 63.],
                              [184., 184., 185., 63.],
                              [192., 193., 193., 63.],
                              [201., 201., 200., 63.],
                              [209., 209., 209., 63.],
                              [217., 217., 218., 63.],
                              [226., 226., 226., 63.],
                              [234., 234., 233., 63.],
                              [242., 242., 242., 63.],
                              [250., 251., 250., 63.]],

                             [[255., 128., 128., 63.],
                              [255., 144., 128., 63.],
                              [255., 169., 128., 63.],
                              [255., 193., 128., 63.],
                              [255., 217., 128., 63.],
                              [255., 242., 128., 63.],
                              [243., 255., 128., 63.],
                              [219., 255., 128., 63.],
                              [195., 255., 128., 63.],
                              [170., 255., 128., 63.],
                              [145., 255., 128., 63.],
                              [128., 255., 133., 63.],
                              [128., 255., 158., 63.],
                              [128., 255., 182., 63.],
                              [128., 255., 207., 63.],
                              [128., 255., 231., 63.],
                              [128., 255., 255., 63.],
                              [128., 230., 255., 63.],
                              [128., 205., 255., 63.],
                              [128., 181., 255., 63.],
                              [128., 156., 255., 63.],
                              [128., 132., 255., 63.],
                              [147., 128., 255., 63.],
                              [171., 128., 255., 63.],
                              [197., 128., 255., 63.],
                              [221., 128., 255., 63.],
                              [245., 128., 255., 63.],
                              [255., 128., 241., 63.],
                              [255., 128., 216., 63.],
                              [255., 128., 191., 63.],
                              [255., 128., 167., 63.],
                              [255., 128., 142., 63.]],

                             [[255., 0., 0., 63.],
                              [255., 34., 0., 63.],
                              [255., 83., 0., 63.],
                              [255., 132., 0., 63.],
                              [255., 180., 0., 63.],
                              [255., 229., 0., 63.],
                              [232., 255., 0., 63.],
                              [184., 255., 0., 63.],
                              [134., 255., 0., 63.],
                              [86., 255., 0., 63.],
                              [36., 255., 0., 63.],
                              [0., 255., 12., 63.],
                              [0., 255., 61., 63.],
                              [0., 255., 109., 63.],
                              [0., 255., 158., 63.],
                              [0., 255., 208., 63.],
                              [0., 254., 255., 63.],
                              [0., 205., 255., 63.],
                              [0., 156., 255., 63.],
                              [0., 107., 255., 63.],
                              [0., 58., 255., 63.],
                              [0., 9., 255., 63.],
                              [40., 0., 255., 63.],
                              [88., 0., 255., 63.],
                              [138., 0., 255., 63.],
                              [186., 0., 255., 63.],
                              [235., 0., 255., 63.],
                              [255., 0., 226., 63.],
                              [255., 0., 177., 63.],
                              [255., 0., 128., 63.],
                              [255., 0., 79., 63.],
                              [255., 0., 30., 63.]],

                             [[142., 0., 0., 63.],
                              [142., 19., 0., 63.],
                              [142., 46., 0., 63.],
                              [142., 74., 0., 63.],
                              [142., 101., 0., 63.],
                              [142., 128., 0., 63.],
                              [130., 142., 0., 63.],
                              [103., 142., 0., 63.],
                              [75., 142., 0., 63.],
                              [48., 142., 0., 63.],
                              [20., 142., 0., 63.],
                              [0., 142., 7., 63.],
                              [0., 142., 34., 63.],
                              [0., 142., 61., 63.],
                              [0., 142., 88., 63.],
                              [0., 142., 116., 63.],
                              [0., 142., 142., 63.],
                              [0., 115., 142., 63.],
                              [0., 87., 142., 63.],
                              [0., 60., 142., 63.],
                              [0., 32., 142., 63.],
                              [0., 5., 142., 63.],
                              [22., 0., 142., 63.],
                              [49., 0., 142., 63.],
                              [77., 0., 142., 63.],
                              [104., 0., 142., 63.],
                              [131., 0., 142., 63.],
                              [142., 0., 126., 63.],
                              [142., 0., 99., 63.],
                              [142., 0., 71., 63.],
                              [142., 0., 44., 63.],
                              [142., 0., 17., 63.]]])

    RAINBOW_LINEAR = np.array([[[0.00000000e+00, 0.00000000e+00, 0.00000000e+00, 1.00000000e+00],
                                [6.07053967e-04, 6.07053967e-04, 6.07053967e-04, 1.00000000e+00],
                                [3.34653576e-03, 3.03526984e-03, 3.03526984e-03, 1.00000000e+00],
                                [6.04883302e-03, 6.51209079e-03, 6.51209079e-03, 1.00000000e+00],
                                [1.09600940e-02, 1.09600940e-02, 1.03298230e-02, 1.00000000e+00],
                                [1.68073758e-02, 1.68073758e-02, 1.68073758e-02, 1.00000000e+00],
                                [2.41576324e-02, 2.41576324e-02, 2.41576324e-02, 1.00000000e+00],
                                [3.43398068e-02, 3.43398068e-02, 3.31047666e-02, 1.00000000e+00],
                                [4.51862044e-02, 4.51862044e-02, 4.51862044e-02, 1.00000000e+00],
                                [5.95112382e-02, 5.95112382e-02, 5.78054302e-02, 1.00000000e+00],
                                [7.42135684e-02, 7.22718507e-02, 7.22718507e-02, 1.00000000e+00],
                                [9.08417112e-02, 9.08417112e-02, 9.08417112e-02, 1.00000000e+00],
                                [1.09461711e-01, 1.09461711e-01, 1.11932428e-01, 1.00000000e+00],
                                [1.32868322e-01, 1.30136477e-01, 1.32868322e-01, 1.00000000e+00],
                                [1.55926464e-01, 1.52926152e-01, 1.55926464e-01, 1.00000000e+00],
                                [1.81164244e-01, 1.81164244e-01, 1.81164244e-01, 1.00000000e+00],
                                [2.08636870e-01, 2.08636870e-01, 2.08636870e-01, 1.00000000e+00],
                                [2.42281122e-01, 2.38397574e-01, 2.38397574e-01, 1.00000000e+00],
                                [2.70497791e-01, 2.74677312e-01, 2.74677312e-01, 1.00000000e+00],
                                [3.09468923e-01, 3.09468923e-01, 3.09468923e-01, 1.00000000e+00],
                                [3.46704056e-01, 3.51532600e-01, 3.46704056e-01, 1.00000000e+00],
                                [3.91572478e-01, 3.91572478e-01, 3.86429434e-01, 1.00000000e+00],
                                [4.34153636e-01, 4.34153636e-01, 4.34153636e-01, 1.00000000e+00],
                                [4.79320183e-01, 4.79320183e-01, 4.85149940e-01, 1.00000000e+00],
                                [5.27115126e-01, 5.33276404e-01, 5.33276404e-01, 1.00000000e+00],
                                [5.84078418e-01, 5.84078418e-01, 5.77580440e-01, 1.00000000e+00],
                                [6.37596874e-01, 6.37596874e-01, 6.37596874e-01, 1.00000000e+00],
                                [6.93871761e-01, 6.93871761e-01, 7.01101892e-01, 1.00000000e+00],
                                [7.60524505e-01, 7.60524505e-01, 7.60524505e-01, 1.00000000e+00],
                                [8.22785754e-01, 8.22785754e-01, 8.14846572e-01, 1.00000000e+00],
                                [8.87923118e-01, 8.87923118e-01, 8.87923118e-01, 1.00000000e+00],
                                [9.55973353e-01, 9.64686248e-01, 9.55973353e-01, 1.00000000e+00]],

                               [[1.00000000e+00, 2.15860500e-01, 2.15860500e-01, 1.00000000e+00],
                                [1.00000000e+00, 2.78894263e-01, 2.15860500e-01, 1.00000000e+00],
                                [1.00000000e+00, 3.96755231e-01, 2.15860500e-01, 1.00000000e+00],
                                [1.00000000e+00, 5.33276404e-01, 2.15860500e-01, 1.00000000e+00],
                                [1.00000000e+00, 6.93871761e-01, 2.15860500e-01, 1.00000000e+00],
                                [1.00000000e+00, 8.87923118e-01, 2.15860500e-01, 1.00000000e+00],
                                [8.96269353e-01, 1.00000000e+00, 2.15860500e-01, 1.00000000e+00],
                                [7.08375780e-01, 1.00000000e+00, 2.15860500e-01, 1.00000000e+00],
                                [5.45724461e-01, 1.00000000e+00, 2.15860500e-01, 1.00000000e+00],
                                [4.01977780e-01, 1.00000000e+00, 2.15860500e-01, 1.00000000e+00],
                                [2.83148740e-01, 1.00000000e+00, 2.15860500e-01, 1.00000000e+00],
                                [2.15860500e-01, 1.00000000e+00, 2.34550582e-01, 1.00000000e+00],
                                [2.15860500e-01, 1.00000000e+00, 3.41914425e-01, 1.00000000e+00],
                                [2.15860500e-01, 1.00000000e+00, 4.67783796e-01, 1.00000000e+00],
                                [2.15860500e-01, 1.00000000e+00, 6.23960392e-01, 1.00000000e+00],
                                [2.15860500e-01, 1.00000000e+00, 7.99102738e-01, 1.00000000e+00],
                                [2.15860500e-01, 1.00000000e+00, 1.00000000e+00, 1.00000000e+00],
                                [2.15860500e-01, 7.91297940e-01, 1.00000000e+00, 1.00000000e+00],
                                [2.15860500e-01, 6.10495571e-01, 1.00000000e+00, 1.00000000e+00],
                                [2.15860500e-01, 4.62077000e-01, 1.00000000e+00, 1.00000000e+00],
                                [2.15860500e-01, 3.32451536e-01, 1.00000000e+00, 1.00000000e+00],
                                [2.15860500e-01, 2.30740049e-01, 1.00000000e+00, 1.00000000e+00],
                                [2.91770650e-01, 2.15860500e-01, 1.00000000e+00, 1.00000000e+00],
                                [4.07240212e-01, 2.15860500e-01, 1.00000000e+00, 1.00000000e+00],
                                [5.58340390e-01, 2.15860500e-01, 1.00000000e+00, 1.00000000e+00],
                                [7.23055129e-01, 2.15860500e-01, 1.00000000e+00, 1.00000000e+00],
                                [9.13098652e-01, 2.15860500e-01, 1.00000000e+00, 1.00000000e+00],
                                [1.00000000e+00, 2.15860500e-01, 8.79622397e-01, 1.00000000e+00],
                                [1.00000000e+00, 2.15860500e-01, 6.86685312e-01, 1.00000000e+00],
                                [1.00000000e+00, 2.15860500e-01, 5.20995573e-01, 1.00000000e+00],
                                [1.00000000e+00, 2.15860500e-01, 3.86429434e-01, 1.00000000e+00],
                                [1.00000000e+00, 2.15860500e-01, 2.70497791e-01, 1.00000000e+00]],

                               [[1.00000000e+00, 0.00000000e+00, 0.00000000e+00, 1.00000000e+00],
                                [1.00000000e+00, 1.59962934e-02, 0.00000000e+00, 1.00000000e+00],
                                [1.00000000e+00, 8.65004620e-02, 0.00000000e+00, 1.00000000e+00],
                                [1.00000000e+00, 2.30740049e-01, 0.00000000e+00, 1.00000000e+00],
                                [1.00000000e+00, 4.56411023e-01, 0.00000000e+00, 1.00000000e+00],
                                [1.00000000e+00, 7.83537792e-01, 0.00000000e+00, 1.00000000e+00],
                                [8.06952258e-01, 1.00000000e+00, 0.00000000e+00, 1.00000000e+00],
                                [4.79320183e-01, 1.00000000e+00, 0.00000000e+00, 1.00000000e+00],
                                [2.38397574e-01, 1.00000000e+00, 0.00000000e+00, 1.00000000e+00],
                                [9.30589628e-02, 1.00000000e+00, 0.00000000e+00, 1.00000000e+00],
                                [1.76419545e-02, 1.00000000e+00, 0.00000000e+00, 1.00000000e+00],
                                [0.00000000e+00, 1.00000000e+00, 3.67650732e-03, 1.00000000e+00],
                                [0.00000000e+00, 1.00000000e+00, 4.66650863e-02, 1.00000000e+00],
                                [0.00000000e+00, 1.00000000e+00, 1.52926152e-01, 1.00000000e+00],
                                [0.00000000e+00, 1.00000000e+00, 3.41914425e-01, 1.00000000e+00],
                                [0.00000000e+00, 1.00000000e+00, 6.30757136e-01, 1.00000000e+00],
                                [0.00000000e+00, 9.91102097e-01, 1.00000000e+00, 1.00000000e+00],
                                [0.00000000e+00, 6.10495571e-01, 1.00000000e+00, 1.00000000e+00],
                                [0.00000000e+00, 3.32451536e-01, 1.00000000e+00, 1.00000000e+00],
                                [0.00000000e+00, 1.47027266e-01, 1.00000000e+00, 1.00000000e+00],
                                [0.00000000e+00, 4.23114106e-02, 1.00000000e+00, 1.00000000e+00],
                                [0.00000000e+00, 2.73174285e-03, 1.00000000e+00, 1.00000000e+00],
                                [2.12190104e-02, 0.00000000e+00, 1.00000000e+00, 1.00000000e+00],
                                [9.75873471e-02, 0.00000000e+00, 1.00000000e+00, 1.00000000e+00],
                                [2.54152094e-01, 0.00000000e+00, 1.00000000e+00, 1.00000000e+00],
                                [4.91020850e-01, 0.00000000e+00, 1.00000000e+00, 1.00000000e+00],
                                [8.30769877e-01, 0.00000000e+00, 1.00000000e+00, 1.00000000e+00],
                                [1.00000000e+00, 0.00000000e+00, 7.60524505e-01, 1.00000000e+00],
                                [1.00000000e+00, 0.00000000e+00, 4.39657174e-01, 1.00000000e+00],
                                [1.00000000e+00, 0.00000000e+00, 2.15860500e-01, 1.00000000e+00],
                                [1.00000000e+00, 0.00000000e+00, 7.81874218e-02, 1.00000000e+00],
                                [1.00000000e+00, 0.00000000e+00, 1.29830323e-02, 1.00000000e+00]],

                               [[2.70497791e-01, 0.00000000e+00, 0.00000000e+00, 1.00000000e+00],
                                [2.70497791e-01, 6.51209079e-03, 0.00000000e+00, 1.00000000e+00],
                                [2.70497791e-01, 2.73208916e-02, 0.00000000e+00, 1.00000000e+00],
                                [2.70497791e-01, 6.84781698e-02, 0.00000000e+00, 1.00000000e+00],
                                [2.70497791e-01, 1.30136477e-01, 0.00000000e+00, 1.00000000e+00],
                                [2.70497791e-01, 2.15860500e-01, 0.00000000e+00, 1.00000000e+00],
                                [2.23227957e-01, 2.70497791e-01, 0.00000000e+00, 1.00000000e+00],
                                [1.35633330e-01, 2.70497791e-01, 0.00000000e+00, 1.00000000e+00],
                                [7.03600957e-02, 2.70497791e-01, 0.00000000e+00, 1.00000000e+00],
                                [2.95568344e-02, 2.70497791e-01, 0.00000000e+00, 1.00000000e+00],
                                [6.99541019e-03, 2.70497791e-01, 0.00000000e+00, 1.00000000e+00],
                                [0.00000000e+00, 2.70497791e-01, 2.12468888e-03, 1.00000000e+00],
                                [0.00000000e+00, 2.70497791e-01, 1.59962934e-02, 1.00000000e+00],
                                [0.00000000e+00, 2.70497791e-01, 4.66650863e-02, 1.00000000e+00],
                                [0.00000000e+00, 2.70497791e-01, 9.75873471e-02, 1.00000000e+00],
                                [0.00000000e+00, 2.70497791e-01, 1.74647404e-01, 1.00000000e+00],
                                [0.00000000e+00, 2.70497791e-01, 2.70497791e-01, 1.00000000e+00],
                                [0.00000000e+00, 1.71441101e-01, 2.70497791e-01, 1.00000000e+00],
                                [0.00000000e+00, 9.53074666e-02, 2.70497791e-01, 1.00000000e+00],
                                [0.00000000e+00, 4.51862044e-02, 2.70497791e-01, 1.00000000e+00],
                                [0.00000000e+00, 1.44438436e-02, 2.70497791e-01, 1.00000000e+00],
                                [0.00000000e+00, 1.51763492e-03, 2.70497791e-01, 1.00000000e+00],
                                [8.02319299e-03, 0.00000000e+00, 2.70497791e-01, 1.00000000e+00],
                                [3.07134437e-02, 0.00000000e+00, 2.70497791e-01, 1.00000000e+00],
                                [7.42135684e-02, 0.00000000e+00, 2.70497791e-01, 1.00000000e+00],
                                [1.38431615e-01, 0.00000000e+00, 2.70497791e-01, 1.00000000e+00],
                                [2.26965874e-01, 0.00000000e+00, 2.70497791e-01, 1.00000000e+00],
                                [2.70497791e-01, 0.00000000e+00, 2.08636870e-01, 1.00000000e+00],
                                [2.70497791e-01, 0.00000000e+00, 1.24771818e-01, 1.00000000e+00],
                                [2.70497791e-01, 0.00000000e+00, 6.30100177e-02, 1.00000000e+00],
                                [2.70497791e-01, 0.00000000e+00, 2.51868596e-02, 1.00000000e+00],
                                [2.70497791e-01, 0.00000000e+00, 5.60539162e-03, 1.00000000e+00]],

                               [[0.00000000e+00, 0.00000000e+00, 0.00000000e+00, 7.49019608e-01],
                                [6.07053967e-04, 6.07053967e-04, 6.07053967e-04, 7.49019608e-01],
                                [3.34653576e-03, 3.03526984e-03, 3.03526984e-03, 7.49019608e-01],
                                [6.04883302e-03, 6.51209079e-03, 6.51209079e-03, 7.49019608e-01],
                                [1.09600940e-02, 1.09600940e-02, 1.03298230e-02, 7.49019608e-01],
                                [1.68073758e-02, 1.68073758e-02, 1.68073758e-02, 7.49019608e-01],
                                [2.41576324e-02, 2.41576324e-02, 2.41576324e-02, 7.49019608e-01],
                                [3.43398068e-02, 3.43398068e-02, 3.31047666e-02, 7.49019608e-01],
                                [4.51862044e-02, 4.51862044e-02, 4.51862044e-02, 7.49019608e-01],
                                [5.95112382e-02, 5.95112382e-02, 5.78054302e-02, 7.49019608e-01],
                                [7.42135684e-02, 7.22718507e-02, 7.22718507e-02, 7.49019608e-01],
                                [9.08417112e-02, 9.08417112e-02, 9.08417112e-02, 7.49019608e-01],
                                [1.09461711e-01, 1.09461711e-01, 1.11932428e-01, 7.49019608e-01],
                                [1.32868322e-01, 1.30136477e-01, 1.32868322e-01, 7.49019608e-01],
                                [1.55926464e-01, 1.52926152e-01, 1.55926464e-01, 7.49019608e-01],
                                [1.81164244e-01, 1.81164244e-01, 1.81164244e-01, 7.49019608e-01],
                                [2.08636870e-01, 2.08636870e-01, 2.08636870e-01, 7.49019608e-01],
                                [2.42281122e-01, 2.38397574e-01, 2.38397574e-01, 7.49019608e-01],
                                [2.70497791e-01, 2.74677312e-01, 2.74677312e-01, 7.49019608e-01],
                                [3.09468923e-01, 3.09468923e-01, 3.09468923e-01, 7.49019608e-01],
                                [3.46704056e-01, 3.51532600e-01, 3.46704056e-01, 7.49019608e-01],
                                [3.91572478e-01, 3.91572478e-01, 3.86429434e-01, 7.49019608e-01],
                                [4.34153636e-01, 4.34153636e-01, 4.34153636e-01, 7.49019608e-01],
                                [4.79320183e-01, 4.79320183e-01, 4.85149940e-01, 7.49019608e-01],
                                [5.27115126e-01, 5.33276404e-01, 5.33276404e-01, 7.49019608e-01],
                                [5.84078418e-01, 5.84078418e-01, 5.77580440e-01, 7.49019608e-01],
                                [6.37596874e-01, 6.37596874e-01, 6.37596874e-01, 7.49019608e-01],
                                [6.93871761e-01, 6.93871761e-01, 7.01101892e-01, 7.49019608e-01],
                                [7.60524505e-01, 7.60524505e-01, 7.60524505e-01, 7.49019608e-01],
                                [8.22785754e-01, 8.22785754e-01, 8.14846572e-01, 7.49019608e-01],
                                [8.87923118e-01, 8.87923118e-01, 8.87923118e-01, 7.49019608e-01],
                                [9.55973353e-01, 9.64686248e-01, 9.55973353e-01, 7.49019608e-01]],

                               [[1.00000000e+00, 2.15860500e-01, 2.15860500e-01, 7.49019608e-01],
                                [1.00000000e+00, 2.78894263e-01, 2.15860500e-01, 7.49019608e-01],
                                [1.00000000e+00, 3.96755231e-01, 2.15860500e-01, 7.49019608e-01],
                                [1.00000000e+00, 5.33276404e-01, 2.15860500e-01, 7.49019608e-01],
                                [1.00000000e+00, 6.93871761e-01, 2.15860500e-01, 7.49019608e-01],
                                [1.00000000e+00, 8.87923118e-01, 2.15860500e-01, 7.49019608e-01],
                                [8.96269353e-01, 1.00000000e+00, 2.15860500e-01, 7.49019608e-01],
                                [7.08375780e-01, 1.00000000e+00, 2.15860500e-01, 7.49019608e-01],
                                [5.45724461e-01, 1.00000000e+00, 2.15860500e-01, 7.49019608e-01],
                                [4.01977780e-01, 1.00000000e+00, 2.15860500e-01, 7.49019608e-01],
                                [2.83148740e-01, 1.00000000e+00, 2.15860500e-01, 7.49019608e-01],
                                [2.15860500e-01, 1.00000000e+00, 2.34550582e-01, 7.49019608e-01],
                                [2.15860500e-01, 1.00000000e+00, 3.41914425e-01, 7.49019608e-01],
                                [2.15860500e-01, 1.00000000e+00, 4.67783796e-01, 7.49019608e-01],
                                [2.15860500e-01, 1.00000000e+00, 6.23960392e-01, 7.49019608e-01],
                                [2.15860500e-01, 1.00000000e+00, 7.99102738e-01, 7.49019608e-01],
                                [2.15860500e-01, 1.00000000e+00, 1.00000000e+00, 7.49019608e-01],
                                [2.15860500e-01, 7.91297940e-01, 1.00000000e+00, 7.49019608e-01],
                                [2.15860500e-01, 6.10495571e-01, 1.00000000e+00, 7.49019608e-01],
                                [2.15860500e-01, 4.62077000e-01, 1.00000000e+00, 7.49019608e-01],
                                [2.15860500e-01, 3.32451536e-01, 1.00000000e+00, 7.49019608e-01],
                                [2.15860500e-01, 2.30740049e-01, 1.00000000e+00, 7.49019608e-01],
                                [2.91770650e-01, 2.15860500e-01, 1.00000000e+00, 7.49019608e-01],
                                [4.07240212e-01, 2.15860500e-01, 1.00000000e+00, 7.49019608e-01],
                                [5.58340390e-01, 2.15860500e-01, 1.00000000e+00, 7.49019608e-01],
                                [7.23055129e-01, 2.15860500e-01, 1.00000000e+00, 7.49019608e-01],
                                [9.13098652e-01, 2.15860500e-01, 1.00000000e+00, 7.49019608e-01],
                                [1.00000000e+00, 2.15860500e-01, 8.79622397e-01, 7.49019608e-01],
                                [1.00000000e+00, 2.15860500e-01, 6.86685312e-01, 7.49019608e-01],
                                [1.00000000e+00, 2.15860500e-01, 5.20995573e-01, 7.49019608e-01],
                                [1.00000000e+00, 2.15860500e-01, 3.86429434e-01, 7.49019608e-01],
                                [1.00000000e+00, 2.15860500e-01, 2.70497791e-01, 7.49019608e-01]],

                               [[1.00000000e+00, 0.00000000e+00, 0.00000000e+00, 7.49019608e-01],
                                [1.00000000e+00, 1.59962934e-02, 0.00000000e+00, 7.49019608e-01],
                                [1.00000000e+00, 8.65004620e-02, 0.00000000e+00, 7.49019608e-01],
                                [1.00000000e+00, 2.30740049e-01, 0.00000000e+00, 7.49019608e-01],
                                [1.00000000e+00, 4.56411023e-01, 0.00000000e+00, 7.49019608e-01],
                                [1.00000000e+00, 7.83537792e-01, 0.00000000e+00, 7.49019608e-01],
                                [8.06952258e-01, 1.00000000e+00, 0.00000000e+00, 7.49019608e-01],
                                [4.79320183e-01, 1.00000000e+00, 0.00000000e+00, 7.49019608e-01],
                                [2.38397574e-01, 1.00000000e+00, 0.00000000e+00, 7.49019608e-01],
                                [9.30589628e-02, 1.00000000e+00, 0.00000000e+00, 7.49019608e-01],
                                [1.76419545e-02, 1.00000000e+00, 0.00000000e+00, 7.49019608e-01],
                                [0.00000000e+00, 1.00000000e+00, 3.67650732e-03, 7.49019608e-01],
                                [0.00000000e+00, 1.00000000e+00, 4.66650863e-02, 7.49019608e-01],
                                [0.00000000e+00, 1.00000000e+00, 1.52926152e-01, 7.49019608e-01],
                                [0.00000000e+00, 1.00000000e+00, 3.41914425e-01, 7.49019608e-01],
                                [0.00000000e+00, 1.00000000e+00, 6.30757136e-01, 7.49019608e-01],
                                [0.00000000e+00, 9.91102097e-01, 1.00000000e+00, 7.49019608e-01],
                                [0.00000000e+00, 6.10495571e-01, 1.00000000e+00, 7.49019608e-01],
                                [0.00000000e+00, 3.32451536e-01, 1.00000000e+00, 7.49019608e-01],
                                [0.00000000e+00, 1.47027266e-01, 1.00000000e+00, 7.49019608e-01],
                                [0.00000000e+00, 4.23114106e-02, 1.00000000e+00, 7.49019608e-01],
                                [0.00000000e+00, 2.73174285e-03, 1.00000000e+00, 7.49019608e-01],
                                [2.12190104e-02, 0.00000000e+00, 1.00000000e+00, 7.49019608e-01],
                                [9.75873471e-02, 0.00000000e+00, 1.00000000e+00, 7.49019608e-01],
                                [2.54152094e-01, 0.00000000e+00, 1.00000000e+00, 7.49019608e-01],
                                [4.91020850e-01, 0.00000000e+00, 1.00000000e+00, 7.49019608e-01],
                                [8.30769877e-01, 0.00000000e+00, 1.00000000e+00, 7.49019608e-01],
                                [1.00000000e+00, 0.00000000e+00, 7.60524505e-01, 7.49019608e-01],
                                [1.00000000e+00, 0.00000000e+00, 4.39657174e-01, 7.49019608e-01],
                                [1.00000000e+00, 0.00000000e+00, 2.15860500e-01, 7.49019608e-01],
                                [1.00000000e+00, 0.00000000e+00, 7.81874218e-02, 7.49019608e-01],
                                [1.00000000e+00, 0.00000000e+00, 1.29830323e-02, 7.49019608e-01]],

                               [[2.70497791e-01, 0.00000000e+00, 0.00000000e+00, 7.49019608e-01],
                                [2.70497791e-01, 6.51209079e-03, 0.00000000e+00, 7.49019608e-01],
                                [2.70497791e-01, 2.73208916e-02, 0.00000000e+00, 7.49019608e-01],
                                [2.70497791e-01, 6.84781698e-02, 0.00000000e+00, 7.49019608e-01],
                                [2.70497791e-01, 1.30136477e-01, 0.00000000e+00, 7.49019608e-01],
                                [2.70497791e-01, 2.15860500e-01, 0.00000000e+00, 7.49019608e-01],
                                [2.23227957e-01, 2.70497791e-01, 0.00000000e+00, 7.49019608e-01],
                                [1.35633330e-01, 2.70497791e-01, 0.00000000e+00, 7.49019608e-01],
                                [7.03600957e-02, 2.70497791e-01, 0.00000000e+00, 7.49019608e-01],
                                [2.95568344e-02, 2.70497791e-01, 0.00000000e+00, 7.49019608e-01],
                                [6.99541019e-03, 2.70497791e-01, 0.00000000e+00, 7.49019608e-01],
                                [0.00000000e+00, 2.70497791e-01, 2.12468888e-03, 7.49019608e-01],
                                [0.00000000e+00, 2.70497791e-01, 1.59962934e-02, 7.49019608e-01],
                                [0.00000000e+00, 2.70497791e-01, 4.66650863e-02, 7.49019608e-01],
                                [0.00000000e+00, 2.70497791e-01, 9.75873471e-02, 7.49019608e-01],
                                [0.00000000e+00, 2.70497791e-01, 1.74647404e-01, 7.49019608e-01],
                                [0.00000000e+00, 2.70497791e-01, 2.70497791e-01, 7.49019608e-01],
                                [0.00000000e+00, 1.71441101e-01, 2.70497791e-01, 7.49019608e-01],
                                [0.00000000e+00, 9.53074666e-02, 2.70497791e-01, 7.49019608e-01],
                                [0.00000000e+00, 4.51862044e-02, 2.70497791e-01, 7.49019608e-01],
                                [0.00000000e+00, 1.44438436e-02, 2.70497791e-01, 7.49019608e-01],
                                [0.00000000e+00, 1.51763492e-03, 2.70497791e-01, 7.49019608e-01],
                                [8.02319299e-03, 0.00000000e+00, 2.70497791e-01, 7.49019608e-01],
                                [3.07134437e-02, 0.00000000e+00, 2.70497791e-01, 7.49019608e-01],
                                [7.42135684e-02, 0.00000000e+00, 2.70497791e-01, 7.49019608e-01],
                                [1.38431615e-01, 0.00000000e+00, 2.70497791e-01, 7.49019608e-01],
                                [2.26965874e-01, 0.00000000e+00, 2.70497791e-01, 7.49019608e-01],
                                [2.70497791e-01, 0.00000000e+00, 2.08636870e-01, 7.49019608e-01],
                                [2.70497791e-01, 0.00000000e+00, 1.24771818e-01, 7.49019608e-01],
                                [2.70497791e-01, 0.00000000e+00, 6.30100177e-02, 7.49019608e-01],
                                [2.70497791e-01, 0.00000000e+00, 2.51868596e-02, 7.49019608e-01],
                                [2.70497791e-01, 0.00000000e+00, 5.60539162e-03, 7.49019608e-01]],

                               [[0.00000000e+00, 0.00000000e+00, 0.00000000e+00, 4.98039216e-01],
                                [6.07053967e-04, 6.07053967e-04, 6.07053967e-04, 4.98039216e-01],
                                [3.34653576e-03, 3.03526984e-03, 3.03526984e-03, 4.98039216e-01],
                                [6.04883302e-03, 6.51209079e-03, 6.51209079e-03, 4.98039216e-01],
                                [1.09600940e-02, 1.09600940e-02, 1.03298230e-02, 4.98039216e-01],
                                [1.68073758e-02, 1.68073758e-02, 1.68073758e-02, 4.98039216e-01],
                                [2.41576324e-02, 2.41576324e-02, 2.41576324e-02, 4.98039216e-01],
                                [3.43398068e-02, 3.43398068e-02, 3.31047666e-02, 4.98039216e-01],
                                [4.51862044e-02, 4.51862044e-02, 4.51862044e-02, 4.98039216e-01],
                                [5.95112382e-02, 5.95112382e-02, 5.78054302e-02, 4.98039216e-01],
                                [7.42135684e-02, 7.22718507e-02, 7.22718507e-02, 4.98039216e-01],
                                [9.08417112e-02, 9.08417112e-02, 9.08417112e-02, 4.98039216e-01],
                                [1.09461711e-01, 1.09461711e-01, 1.11932428e-01, 4.98039216e-01],
                                [1.32868322e-01, 1.30136477e-01, 1.32868322e-01, 4.98039216e-01],
                                [1.55926464e-01, 1.52926152e-01, 1.55926464e-01, 4.98039216e-01],
                                [1.81164244e-01, 1.81164244e-01, 1.81164244e-01, 4.98039216e-01],
                                [2.08636870e-01, 2.08636870e-01, 2.08636870e-01, 4.98039216e-01],
                                [2.42281122e-01, 2.38397574e-01, 2.38397574e-01, 4.98039216e-01],
                                [2.70497791e-01, 2.74677312e-01, 2.74677312e-01, 4.98039216e-01],
                                [3.09468923e-01, 3.09468923e-01, 3.09468923e-01, 4.98039216e-01],
                                [3.46704056e-01, 3.51532600e-01, 3.46704056e-01, 4.98039216e-01],
                                [3.91572478e-01, 3.91572478e-01, 3.86429434e-01, 4.98039216e-01],
                                [4.34153636e-01, 4.34153636e-01, 4.34153636e-01, 4.98039216e-01],
                                [4.79320183e-01, 4.79320183e-01, 4.85149940e-01, 4.98039216e-01],
                                [5.27115126e-01, 5.33276404e-01, 5.33276404e-01, 4.98039216e-01],
                                [5.84078418e-01, 5.84078418e-01, 5.77580440e-01, 4.98039216e-01],
                                [6.37596874e-01, 6.37596874e-01, 6.37596874e-01, 4.98039216e-01],
                                [6.93871761e-01, 6.93871761e-01, 7.01101892e-01, 4.98039216e-01],
                                [7.60524505e-01, 7.60524505e-01, 7.60524505e-01, 4.98039216e-01],
                                [8.22785754e-01, 8.22785754e-01, 8.14846572e-01, 4.98039216e-01],
                                [8.87923118e-01, 8.87923118e-01, 8.87923118e-01, 4.98039216e-01],
                                [9.55973353e-01, 9.64686248e-01, 9.55973353e-01, 4.98039216e-01]],

                               [[1.00000000e+00, 2.15860500e-01, 2.15860500e-01, 4.98039216e-01],
                                [1.00000000e+00, 2.78894263e-01, 2.15860500e-01, 4.98039216e-01],
                                [1.00000000e+00, 3.96755231e-01, 2.15860500e-01, 4.98039216e-01],
                                [1.00000000e+00, 5.33276404e-01, 2.15860500e-01, 4.98039216e-01],
                                [1.00000000e+00, 6.93871761e-01, 2.15860500e-01, 4.98039216e-01],
                                [1.00000000e+00, 8.87923118e-01, 2.15860500e-01, 4.98039216e-01],
                                [8.96269353e-01, 1.00000000e+00, 2.15860500e-01, 4.98039216e-01],
                                [7.08375780e-01, 1.00000000e+00, 2.15860500e-01, 4.98039216e-01],
                                [5.45724461e-01, 1.00000000e+00, 2.15860500e-01, 4.98039216e-01],
                                [4.01977780e-01, 1.00000000e+00, 2.15860500e-01, 4.98039216e-01],
                                [2.83148740e-01, 1.00000000e+00, 2.15860500e-01, 4.98039216e-01],
                                [2.15860500e-01, 1.00000000e+00, 2.34550582e-01, 4.98039216e-01],
                                [2.15860500e-01, 1.00000000e+00, 3.41914425e-01, 4.98039216e-01],
                                [2.15860500e-01, 1.00000000e+00, 4.67783796e-01, 4.98039216e-01],
                                [2.15860500e-01, 1.00000000e+00, 6.23960392e-01, 4.98039216e-01],
                                [2.15860500e-01, 1.00000000e+00, 7.99102738e-01, 4.98039216e-01],
                                [2.15860500e-01, 1.00000000e+00, 1.00000000e+00, 4.98039216e-01],
                                [2.15860500e-01, 7.91297940e-01, 1.00000000e+00, 4.98039216e-01],
                                [2.15860500e-01, 6.10495571e-01, 1.00000000e+00, 4.98039216e-01],
                                [2.15860500e-01, 4.62077000e-01, 1.00000000e+00, 4.98039216e-01],
                                [2.15860500e-01, 3.32451536e-01, 1.00000000e+00, 4.98039216e-01],
                                [2.15860500e-01, 2.30740049e-01, 1.00000000e+00, 4.98039216e-01],
                                [2.91770650e-01, 2.15860500e-01, 1.00000000e+00, 4.98039216e-01],
                                [4.07240212e-01, 2.15860500e-01, 1.00000000e+00, 4.98039216e-01],
                                [5.58340390e-01, 2.15860500e-01, 1.00000000e+00, 4.98039216e-01],
                                [7.23055129e-01, 2.15860500e-01, 1.00000000e+00, 4.98039216e-01],
                                [9.13098652e-01, 2.15860500e-01, 1.00000000e+00, 4.98039216e-01],
                                [1.00000000e+00, 2.15860500e-01, 8.79622397e-01, 4.98039216e-01],
                                [1.00000000e+00, 2.15860500e-01, 6.86685312e-01, 4.98039216e-01],
                                [1.00000000e+00, 2.15860500e-01, 5.20995573e-01, 4.98039216e-01],
                                [1.00000000e+00, 2.15860500e-01, 3.86429434e-01, 4.98039216e-01],
                                [1.00000000e+00, 2.15860500e-01, 2.70497791e-01, 4.98039216e-01]],

                               [[1.00000000e+00, 0.00000000e+00, 0.00000000e+00, 4.98039216e-01],
                                [1.00000000e+00, 1.59962934e-02, 0.00000000e+00, 4.98039216e-01],
                                [1.00000000e+00, 8.65004620e-02, 0.00000000e+00, 4.98039216e-01],
                                [1.00000000e+00, 2.30740049e-01, 0.00000000e+00, 4.98039216e-01],
                                [1.00000000e+00, 4.56411023e-01, 0.00000000e+00, 4.98039216e-01],
                                [1.00000000e+00, 7.83537792e-01, 0.00000000e+00, 4.98039216e-01],
                                [8.06952258e-01, 1.00000000e+00, 0.00000000e+00, 4.98039216e-01],
                                [4.79320183e-01, 1.00000000e+00, 0.00000000e+00, 4.98039216e-01],
                                [2.38397574e-01, 1.00000000e+00, 0.00000000e+00, 4.98039216e-01],
                                [9.30589628e-02, 1.00000000e+00, 0.00000000e+00, 4.98039216e-01],
                                [1.76419545e-02, 1.00000000e+00, 0.00000000e+00, 4.98039216e-01],
                                [0.00000000e+00, 1.00000000e+00, 3.67650732e-03, 4.98039216e-01],
                                [0.00000000e+00, 1.00000000e+00, 4.66650863e-02, 4.98039216e-01],
                                [0.00000000e+00, 1.00000000e+00, 1.52926152e-01, 4.98039216e-01],
                                [0.00000000e+00, 1.00000000e+00, 3.41914425e-01, 4.98039216e-01],
                                [0.00000000e+00, 1.00000000e+00, 6.30757136e-01, 4.98039216e-01],
                                [0.00000000e+00, 9.91102097e-01, 1.00000000e+00, 4.98039216e-01],
                                [0.00000000e+00, 6.10495571e-01, 1.00000000e+00, 4.98039216e-01],
                                [0.00000000e+00, 3.32451536e-01, 1.00000000e+00, 4.98039216e-01],
                                [0.00000000e+00, 1.47027266e-01, 1.00000000e+00, 4.98039216e-01],
                                [0.00000000e+00, 4.23114106e-02, 1.00000000e+00, 4.98039216e-01],
                                [0.00000000e+00, 2.73174285e-03, 1.00000000e+00, 4.98039216e-01],
                                [2.12190104e-02, 0.00000000e+00, 1.00000000e+00, 4.98039216e-01],
                                [9.75873471e-02, 0.00000000e+00, 1.00000000e+00, 4.98039216e-01],
                                [2.54152094e-01, 0.00000000e+00, 1.00000000e+00, 4.98039216e-01],
                                [4.91020850e-01, 0.00000000e+00, 1.00000000e+00, 4.98039216e-01],
                                [8.30769877e-01, 0.00000000e+00, 1.00000000e+00, 4.98039216e-01],
                                [1.00000000e+00, 0.00000000e+00, 7.60524505e-01, 4.98039216e-01],
                                [1.00000000e+00, 0.00000000e+00, 4.39657174e-01, 4.98039216e-01],
                                [1.00000000e+00, 0.00000000e+00, 2.15860500e-01, 4.98039216e-01],
                                [1.00000000e+00, 0.00000000e+00, 7.81874218e-02, 4.98039216e-01],
                                [1.00000000e+00, 0.00000000e+00, 1.29830323e-02, 4.98039216e-01]],

                               [[2.70497791e-01, 0.00000000e+00, 0.00000000e+00, 4.98039216e-01],
                                [2.70497791e-01, 6.51209079e-03, 0.00000000e+00, 4.98039216e-01],
                                [2.70497791e-01, 2.73208916e-02, 0.00000000e+00, 4.98039216e-01],
                                [2.70497791e-01, 6.84781698e-02, 0.00000000e+00, 4.98039216e-01],
                                [2.70497791e-01, 1.30136477e-01, 0.00000000e+00, 4.98039216e-01],
                                [2.70497791e-01, 2.15860500e-01, 0.00000000e+00, 4.98039216e-01],
                                [2.23227957e-01, 2.70497791e-01, 0.00000000e+00, 4.98039216e-01],
                                [1.35633330e-01, 2.70497791e-01, 0.00000000e+00, 4.98039216e-01],
                                [7.03600957e-02, 2.70497791e-01, 0.00000000e+00, 4.98039216e-01],
                                [2.95568344e-02, 2.70497791e-01, 0.00000000e+00, 4.98039216e-01],
                                [6.99541019e-03, 2.70497791e-01, 0.00000000e+00, 4.98039216e-01],
                                [0.00000000e+00, 2.70497791e-01, 2.12468888e-03, 4.98039216e-01],
                                [0.00000000e+00, 2.70497791e-01, 1.59962934e-02, 4.98039216e-01],
                                [0.00000000e+00, 2.70497791e-01, 4.66650863e-02, 4.98039216e-01],
                                [0.00000000e+00, 2.70497791e-01, 9.75873471e-02, 4.98039216e-01],
                                [0.00000000e+00, 2.70497791e-01, 1.74647404e-01, 4.98039216e-01],
                                [0.00000000e+00, 2.70497791e-01, 2.70497791e-01, 4.98039216e-01],
                                [0.00000000e+00, 1.71441101e-01, 2.70497791e-01, 4.98039216e-01],
                                [0.00000000e+00, 9.53074666e-02, 2.70497791e-01, 4.98039216e-01],
                                [0.00000000e+00, 4.51862044e-02, 2.70497791e-01, 4.98039216e-01],
                                [0.00000000e+00, 1.44438436e-02, 2.70497791e-01, 4.98039216e-01],
                                [0.00000000e+00, 1.51763492e-03, 2.70497791e-01, 4.98039216e-01],
                                [8.02319299e-03, 0.00000000e+00, 2.70497791e-01, 4.98039216e-01],
                                [3.07134437e-02, 0.00000000e+00, 2.70497791e-01, 4.98039216e-01],
                                [7.42135684e-02, 0.00000000e+00, 2.70497791e-01, 4.98039216e-01],
                                [1.38431615e-01, 0.00000000e+00, 2.70497791e-01, 4.98039216e-01],
                                [2.26965874e-01, 0.00000000e+00, 2.70497791e-01, 4.98039216e-01],
                                [2.70497791e-01, 0.00000000e+00, 2.08636870e-01, 4.98039216e-01],
                                [2.70497791e-01, 0.00000000e+00, 1.24771818e-01, 4.98039216e-01],
                                [2.70497791e-01, 0.00000000e+00, 6.30100177e-02, 4.98039216e-01],
                                [2.70497791e-01, 0.00000000e+00, 2.51868596e-02, 4.98039216e-01],
                                [2.70497791e-01, 0.00000000e+00, 5.60539162e-03, 4.98039216e-01]],

                               [[0.00000000e+00, 0.00000000e+00, 0.00000000e+00, 2.47058824e-01],
                                [6.07053967e-04, 6.07053967e-04, 6.07053967e-04, 2.47058824e-01],
                                [3.34653576e-03, 3.03526984e-03, 3.03526984e-03, 2.47058824e-01],
                                [6.04883302e-03, 6.51209079e-03, 6.51209079e-03, 2.47058824e-01],
                                [1.09600940e-02, 1.09600940e-02, 1.03298230e-02, 2.47058824e-01],
                                [1.68073758e-02, 1.68073758e-02, 1.68073758e-02, 2.47058824e-01],
                                [2.41576324e-02, 2.41576324e-02, 2.41576324e-02, 2.47058824e-01],
                                [3.43398068e-02, 3.43398068e-02, 3.31047666e-02, 2.47058824e-01],
                                [4.51862044e-02, 4.51862044e-02, 4.51862044e-02, 2.47058824e-01],
                                [5.95112382e-02, 5.95112382e-02, 5.78054302e-02, 2.47058824e-01],
                                [7.42135684e-02, 7.22718507e-02, 7.22718507e-02, 2.47058824e-01],
                                [9.08417112e-02, 9.08417112e-02, 9.08417112e-02, 2.47058824e-01],
                                [1.09461711e-01, 1.09461711e-01, 1.11932428e-01, 2.47058824e-01],
                                [1.32868322e-01, 1.30136477e-01, 1.32868322e-01, 2.47058824e-01],
                                [1.55926464e-01, 1.52926152e-01, 1.55926464e-01, 2.47058824e-01],
                                [1.81164244e-01, 1.81164244e-01, 1.81164244e-01, 2.47058824e-01],
                                [2.08636870e-01, 2.08636870e-01, 2.08636870e-01, 2.47058824e-01],
                                [2.42281122e-01, 2.38397574e-01, 2.38397574e-01, 2.47058824e-01],
                                [2.70497791e-01, 2.74677312e-01, 2.74677312e-01, 2.47058824e-01],
                                [3.09468923e-01, 3.09468923e-01, 3.09468923e-01, 2.47058824e-01],
                                [3.46704056e-01, 3.51532600e-01, 3.46704056e-01, 2.47058824e-01],
                                [3.91572478e-01, 3.91572478e-01, 3.86429434e-01, 2.47058824e-01],
                                [4.34153636e-01, 4.34153636e-01, 4.34153636e-01, 2.47058824e-01],
                                [4.79320183e-01, 4.79320183e-01, 4.85149940e-01, 2.47058824e-01],
                                [5.27115126e-01, 5.33276404e-01, 5.33276404e-01, 2.47058824e-01],
                                [5.84078418e-01, 5.84078418e-01, 5.77580440e-01, 2.47058824e-01],
                                [6.37596874e-01, 6.37596874e-01, 6.37596874e-01, 2.47058824e-01],
                                [6.93871761e-01, 6.93871761e-01, 7.01101892e-01, 2.47058824e-01],
                                [7.60524505e-01, 7.60524505e-01, 7.60524505e-01, 2.47058824e-01],
                                [8.22785754e-01, 8.22785754e-01, 8.14846572e-01, 2.47058824e-01],
                                [8.87923118e-01, 8.87923118e-01, 8.87923118e-01, 2.47058824e-01],
                                [9.55973353e-01, 9.64686248e-01, 9.55973353e-01, 2.47058824e-01]],

                               [[1.00000000e+00, 2.15860500e-01, 2.15860500e-01, 2.47058824e-01],
                                [1.00000000e+00, 2.78894263e-01, 2.15860500e-01, 2.47058824e-01],
                                [1.00000000e+00, 3.96755231e-01, 2.15860500e-01, 2.47058824e-01],
                                [1.00000000e+00, 5.33276404e-01, 2.15860500e-01, 2.47058824e-01],
                                [1.00000000e+00, 6.93871761e-01, 2.15860500e-01, 2.47058824e-01],
                                [1.00000000e+00, 8.87923118e-01, 2.15860500e-01, 2.47058824e-01],
                                [8.96269353e-01, 1.00000000e+00, 2.15860500e-01, 2.47058824e-01],
                                [7.08375780e-01, 1.00000000e+00, 2.15860500e-01, 2.47058824e-01],
                                [5.45724461e-01, 1.00000000e+00, 2.15860500e-01, 2.47058824e-01],
                                [4.01977780e-01, 1.00000000e+00, 2.15860500e-01, 2.47058824e-01],
                                [2.83148740e-01, 1.00000000e+00, 2.15860500e-01, 2.47058824e-01],
                                [2.15860500e-01, 1.00000000e+00, 2.34550582e-01, 2.47058824e-01],
                                [2.15860500e-01, 1.00000000e+00, 3.41914425e-01, 2.47058824e-01],
                                [2.15860500e-01, 1.00000000e+00, 4.67783796e-01, 2.47058824e-01],
                                [2.15860500e-01, 1.00000000e+00, 6.23960392e-01, 2.47058824e-01],
                                [2.15860500e-01, 1.00000000e+00, 7.99102738e-01, 2.47058824e-01],
                                [2.15860500e-01, 1.00000000e+00, 1.00000000e+00, 2.47058824e-01],
                                [2.15860500e-01, 7.91297940e-01, 1.00000000e+00, 2.47058824e-01],
                                [2.15860500e-01, 6.10495571e-01, 1.00000000e+00, 2.47058824e-01],
                                [2.15860500e-01, 4.62077000e-01, 1.00000000e+00, 2.47058824e-01],
                                [2.15860500e-01, 3.32451536e-01, 1.00000000e+00, 2.47058824e-01],
                                [2.15860500e-01, 2.30740049e-01, 1.00000000e+00, 2.47058824e-01],
                                [2.91770650e-01, 2.15860500e-01, 1.00000000e+00, 2.47058824e-01],
                                [4.07240212e-01, 2.15860500e-01, 1.00000000e+00, 2.47058824e-01],
                                [5.58340390e-01, 2.15860500e-01, 1.00000000e+00, 2.47058824e-01],
                                [7.23055129e-01, 2.15860500e-01, 1.00000000e+00, 2.47058824e-01],
                                [9.13098652e-01, 2.15860500e-01, 1.00000000e+00, 2.47058824e-01],
                                [1.00000000e+00, 2.15860500e-01, 8.79622397e-01, 2.47058824e-01],
                                [1.00000000e+00, 2.15860500e-01, 6.86685312e-01, 2.47058824e-01],
                                [1.00000000e+00, 2.15860500e-01, 5.20995573e-01, 2.47058824e-01],
                                [1.00000000e+00, 2.15860500e-01, 3.86429434e-01, 2.47058824e-01],
                                [1.00000000e+00, 2.15860500e-01, 2.70497791e-01, 2.47058824e-01]],

                               [[1.00000000e+00, 0.00000000e+00, 0.00000000e+00, 2.47058824e-01],
                                [1.00000000e+00, 1.59962934e-02, 0.00000000e+00, 2.47058824e-01],
                                [1.00000000e+00, 8.65004620e-02, 0.00000000e+00, 2.47058824e-01],
                                [1.00000000e+00, 2.30740049e-01, 0.00000000e+00, 2.47058824e-01],
                                [1.00000000e+00, 4.56411023e-01, 0.00000000e+00, 2.47058824e-01],
                                [1.00000000e+00, 7.83537792e-01, 0.00000000e+00, 2.47058824e-01],
                                [8.06952258e-01, 1.00000000e+00, 0.00000000e+00, 2.47058824e-01],
                                [4.79320183e-01, 1.00000000e+00, 0.00000000e+00, 2.47058824e-01],
                                [2.38397574e-01, 1.00000000e+00, 0.00000000e+00, 2.47058824e-01],
                                [9.30589628e-02, 1.00000000e+00, 0.00000000e+00, 2.47058824e-01],
                                [1.76419545e-02, 1.00000000e+00, 0.00000000e+00, 2.47058824e-01],
                                [0.00000000e+00, 1.00000000e+00, 3.67650732e-03, 2.47058824e-01],
                                [0.00000000e+00, 1.00000000e+00, 4.66650863e-02, 2.47058824e-01],
                                [0.00000000e+00, 1.00000000e+00, 1.52926152e-01, 2.47058824e-01],
                                [0.00000000e+00, 1.00000000e+00, 3.41914425e-01, 2.47058824e-01],
                                [0.00000000e+00, 1.00000000e+00, 6.30757136e-01, 2.47058824e-01],
                                [0.00000000e+00, 9.91102097e-01, 1.00000000e+00, 2.47058824e-01],
                                [0.00000000e+00, 6.10495571e-01, 1.00000000e+00, 2.47058824e-01],
                                [0.00000000e+00, 3.32451536e-01, 1.00000000e+00, 2.47058824e-01],
                                [0.00000000e+00, 1.47027266e-01, 1.00000000e+00, 2.47058824e-01],
                                [0.00000000e+00, 4.23114106e-02, 1.00000000e+00, 2.47058824e-01],
                                [0.00000000e+00, 2.73174285e-03, 1.00000000e+00, 2.47058824e-01],
                                [2.12190104e-02, 0.00000000e+00, 1.00000000e+00, 2.47058824e-01],
                                [9.75873471e-02, 0.00000000e+00, 1.00000000e+00, 2.47058824e-01],
                                [2.54152094e-01, 0.00000000e+00, 1.00000000e+00, 2.47058824e-01],
                                [4.91020850e-01, 0.00000000e+00, 1.00000000e+00, 2.47058824e-01],
                                [8.30769877e-01, 0.00000000e+00, 1.00000000e+00, 2.47058824e-01],
                                [1.00000000e+00, 0.00000000e+00, 7.60524505e-01, 2.47058824e-01],
                                [1.00000000e+00, 0.00000000e+00, 4.39657174e-01, 2.47058824e-01],
                                [1.00000000e+00, 0.00000000e+00, 2.15860500e-01, 2.47058824e-01],
                                [1.00000000e+00, 0.00000000e+00, 7.81874218e-02, 2.47058824e-01],
                                [1.00000000e+00, 0.00000000e+00, 1.29830323e-02, 2.47058824e-01]],

                               [[2.70497791e-01, 0.00000000e+00, 0.00000000e+00, 2.47058824e-01],
                                [2.70497791e-01, 6.51209079e-03, 0.00000000e+00, 2.47058824e-01],
                                [2.70497791e-01, 2.73208916e-02, 0.00000000e+00, 2.47058824e-01],
                                [2.70497791e-01, 6.84781698e-02, 0.00000000e+00, 2.47058824e-01],
                                [2.70497791e-01, 1.30136477e-01, 0.00000000e+00, 2.47058824e-01],
                                [2.70497791e-01, 2.15860500e-01, 0.00000000e+00, 2.47058824e-01],
                                [2.23227957e-01, 2.70497791e-01, 0.00000000e+00, 2.47058824e-01],
                                [1.35633330e-01, 2.70497791e-01, 0.00000000e+00, 2.47058824e-01],
                                [7.03600957e-02, 2.70497791e-01, 0.00000000e+00, 2.47058824e-01],
                                [2.95568344e-02, 2.70497791e-01, 0.00000000e+00, 2.47058824e-01],
                                [6.99541019e-03, 2.70497791e-01, 0.00000000e+00, 2.47058824e-01],
                                [0.00000000e+00, 2.70497791e-01, 2.12468888e-03, 2.47058824e-01],
                                [0.00000000e+00, 2.70497791e-01, 1.59962934e-02, 2.47058824e-01],
                                [0.00000000e+00, 2.70497791e-01, 4.66650863e-02, 2.47058824e-01],
                                [0.00000000e+00, 2.70497791e-01, 9.75873471e-02, 2.47058824e-01],
                                [0.00000000e+00, 2.70497791e-01, 1.74647404e-01, 2.47058824e-01],
                                [0.00000000e+00, 2.70497791e-01, 2.70497791e-01, 2.47058824e-01],
                                [0.00000000e+00, 1.71441101e-01, 2.70497791e-01, 2.47058824e-01],
                                [0.00000000e+00, 9.53074666e-02, 2.70497791e-01, 2.47058824e-01],
                                [0.00000000e+00, 4.51862044e-02, 2.70497791e-01, 2.47058824e-01],
                                [0.00000000e+00, 1.44438436e-02, 2.70497791e-01, 2.47058824e-01],
                                [0.00000000e+00, 1.51763492e-03, 2.70497791e-01, 2.47058824e-01],
                                [8.02319299e-03, 0.00000000e+00, 2.70497791e-01, 2.47058824e-01],
                                [3.07134437e-02, 0.00000000e+00, 2.70497791e-01, 2.47058824e-01],
                                [7.42135684e-02, 0.00000000e+00, 2.70497791e-01, 2.47058824e-01],
                                [1.38431615e-01, 0.00000000e+00, 2.70497791e-01, 2.47058824e-01],
                                [2.26965874e-01, 0.00000000e+00, 2.70497791e-01, 2.47058824e-01],
                                [2.70497791e-01, 0.00000000e+00, 2.08636870e-01, 2.47058824e-01],
                                [2.70497791e-01, 0.00000000e+00, 1.24771818e-01, 2.47058824e-01],
                                [2.70497791e-01, 0.00000000e+00, 6.30100177e-02, 2.47058824e-01],
                                [2.70497791e-01, 0.00000000e+00, 2.51868596e-02, 2.47058824e-01],
                                [2.70497791e-01, 0.00000000e+00, 5.60539162e-03, 2.47058824e-01]]])

    def test_white_to_linear(self):
        expected = np.full((1, 1, 4), 1.0)
        r = np.full((1, 1), 255)
        g = np.full((1, 1), 255)
        b = np.full((1, 1), 255)
        a = np.full((1, 1), 255)
        result = to_linear(r, g, b, a)
        assert expected.shape == result.shape
        assert np.allclose(expected, result)

    def test_black_to_linear(self):
        expected = np.full((1, 1, 4), 0.0)
        r = np.full((1, 1), 0)
        g = np.full((1, 1), 0)
        b = np.full((1, 1), 0)
        a = np.full((1, 1), 0)
        result = to_linear(r, g, b, a)
        assert expected.shape == result.shape
        assert np.allclose(expected, result)

    def test_grey_to_linear(self):
        expected = np.array([[[
            0.212,
            0.212,
            0.212,
            0.5
        ]]])
        r = np.full((1, 1), 127)
        g = np.full((1, 1), 127)
        b = np.full((1, 1), 127)
        a = np.full((1, 1), 127)
        result = to_linear(r, g, b, a)
        assert expected.shape == result.shape
        assert np.allclose(expected, result, atol=0.01)

    def test_rainbow_to_linear(self):
        im = self.RAINBOW_SRGB
        expected = self.RAINBOW_LINEAR
        result = to_linear(im[:, :, 0], im[:, :, 1], im[:, :, 2], im[:, :, 3])
        assert expected.shape == result.shape
        assert np.allclose(expected, result, atol=0.00001)

    def test_rainbow_from_linear(self):
        source = self.RAINBOW_LINEAR
        expected = self.RAINBOW_SRGB
        result = from_linear(source)
        assert expected.shape == result.shape
        assert np.allclose(expected, result, atol=0.00001)

    def test_rainbow(self):
        im = self.RAINBOW_SRGB
        im_linear = to_linear(im[:, :, 0], im[:, :, 1], im[:, :, 2], im[:, :, 3])
        result = from_linear(im_linear)
        assert im.shape == result.shape
        assert np.allclose(im, result, atol=0.00001)
